@@ -141,6 +141,10 @@ class Sim:
 
         return event
 
+    async def request_data_once(self, fields: typing.List[DataField], object_id: int = 0, timeout: typing.Optional[float] = None):
+        request = self.request_data_on_sim_object(fields, PERIOD.ONCE, object_id)
+        return await request.wait(timeout)
+
     def request_data_on_sim_object(self, fields: typing.List[DataField], period: PERIOD, object_id: int = 0):
         # TODO: Add explicit unsubscribe
         definition_id = self._definition_id_inc.next(self._data_definition_ids)
@@ -157,7 +161,7 @@ class Sim:
 
         with self._lock:
             for field in fields:
-                self.sim.add_to_data_definition(definition_id, field.name, field.units)
+                self.sim.add_to_data_definition(definition_id, field.name, field.units, field.type.data_type)
                 packet_ids.append(self.sim.get_last_sent_packet_id())
             self.sim.request_data_on_sim_object(request_id, definition_id, object_id, period)
             data_request = DataRequest(
@@ -202,27 +206,28 @@ class Sim:
         data_request.send_data(msg)
 
     def _loop(self) -> None:
-        while not self._stopping:
-            msg = self.sim.receive()
-            base = msg.as_base()
-            if base is None:
-                continue
-            msg_type = _native.RECV_ID(base.dwID)
-            if msg_type == _native.RECV_ID.OPEN:
-                self.open_msg = msg.as_open()
-                self._open_evt.set()
-            elif msg_type == _native.RECV_ID.EVENT:
-                self._process_event(msg.as_event())
-            elif msg_type == _native.RECV_ID.EVENT_FRAME:
-                self._process_event(msg.as_event())
-            elif msg_type == _native.RECV_ID.SIMOBJECT_DATA:
-                self._process_data(msg.as_simobject_data())
-            elif msg_type == _native.RECV_ID.EXCEPTION:
-                self._process_exception(msg.as_exception())
+        try:
+            while not self._stopping:
+                msg = self.sim.receive()
+                base = msg.as_base()
+                if base is None:
+                    continue
+                msg_type = _native.RECV_ID(base.dwID)
+                if msg_type == _native.RECV_ID.OPEN:
+                    self.open_msg = msg.as_open()
+                    self._open_evt.set()
+                elif msg_type == _native.RECV_ID.EVENT:
+                    self._process_event(msg.as_event())
+                elif msg_type == _native.RECV_ID.EVENT_FRAME:
+                    self._process_event(msg.as_event())
+                elif msg_type == _native.RECV_ID.SIMOBJECT_DATA:
+                    self._process_data(msg.as_simobject_data())
+                elif msg_type == _native.RECV_ID.EXCEPTION:
+                    self._process_exception(msg.as_exception())
 
-            # print(_native.RECV_ID(base.dwID))
-
-        self._loop_done_evt.set()
+                # print(_native.RECV_ID(base.dwID))
+        finally:
+            self._loop_done_evt.set()
 
     ########################
     # Synchronous interface
@@ -240,6 +245,10 @@ class Sim:
     def sync_wait_open(self, timeout: typing.Optional[float] = OPEN_TIMEOUT_SEC) -> None:
         return asyncio.run(self.wait_open(timeout))
 
+    @no_async
+    def sync_request_data_once(self, fields: typing.List[DataField], object_id: int = 0, timeout: typing.Optional[float] = None):
+        return asyncio.run(self.request_data_once(fields, object_id, timeout))
+
     @no_async
     def sync_start(self, open_timeout: typing.Optional[float] = OPEN_TIMEOUT_SEC):
         return asyncio.run(self.start(open_timeout))
