@@ -13,7 +13,7 @@ T = typing.TypeVar("T")
 @dataclasses.dataclass  # For smoother inheritance
 class AsyncNotifierThreadSafe:
     _waiters: "deque[asyncio.Future]" = dataclasses.field(default_factory=deque, init=False)
-    _lock: threading.Lock = dataclasses.field(default_factory=threading.Lock, init=False)  # Use in child classes for synchronisation
+    _lock: threading.Lock = dataclasses.field(default_factory=threading.RLock, init=False)  # Use in child classes for synchronisation
 
     def add_waiter(self) -> asyncio.Future:
         waiter = asyncio.get_event_loop().create_future()
@@ -70,7 +70,7 @@ class SimException(Exception):
 class HasExceptions(AsyncNotifierThreadSafe):
     packet_ids: typing.List[int]
 
-    _exception: typing.Optional[SimException] = dataclasses.field(default=None, init=False)
+    _exception: typing.Optional[Exception] = dataclasses.field(default=None, init=False)
 
     def set_exception(self, msg_exc: _native.MsgException):
         assert msg_exc.dwSendID in self.packet_ids
@@ -78,6 +78,11 @@ class HasExceptions(AsyncNotifierThreadSafe):
             self._exception = SimException(msg_exc)
             self.notify_all()
 
+    def set_python_exception(self, exc: Exception):
+        with self._lock:
+            self._exception = exc
+            self.notify_all()
+
     def check_exception(self):
         if self._exception:
             raise self._exception
